package com.censoft.censoftrongtong.controller;

import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.censoft.censoftrongtong.domain.*;
import com.censoft.censoftrongtong.domain.dto.*;
import com.censoft.censoftrongtong.enums.RiskPlanStatusType;
import com.censoft.censoftrongtong.service.*;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.config.RuoYiConfig;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.domain.R;
import com.ruoyi.common.core.domain.entity.SysDept;
import com.ruoyi.common.core.domain.entity.SysDictData;
import com.ruoyi.common.core.domain.entity.SysUser;
import com.ruoyi.common.core.page.TableDataInfo;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.common.utils.bean.BeanUtils;
import com.ruoyi.system.service.ISysDeptService;
import com.ruoyi.system.service.ISysDictDataService;
import com.ruoyi.system.service.ISysUploadFileService;
import com.ruoyi.system.service.ISysUserService;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;


/**
 * 计划app端接口
 *
 * @author censoft
 * @date 2023-06-27
 * @module 融通项目
 */
@RestController
@RequestMapping("/app-api/risk/plan")
public class RiskPlanAppController extends BaseController {

    @Resource
    private ISysDeptService sysDeptService;

    @Resource
    private ILedgerProjectService ledgerProjectService;

    @Resource
    private ILedgerBuildingService ledgerBuildingService;

    @Resource
    private ILedgerFloorService ledgerFloorService;

    @Resource
    private ILedgerRoomService ledgerRoomService;

    @Resource
    private IRiskPlanService riskPlanService;

    @Resource
    private IRiskPlanInherentListService riskPlanInherentListService;

    @Resource
    private IRiskPlanExistingListService riskPlanExistingListService;

    @Resource
    private ISysUploadFileService uploadFileService;

    @Resource
    private ILawListService lawListService;

    @Resource
    private ISysDictDataService dictDataService;

    @Resource
    private IRiskPlanUserBuildingService riskPlanUserBuildingService;

    @Resource
    private IRiskAssessmentMatrixScoreService riskAssessmentMatrixScoreService;

    @Resource
    private IRiskAssessmentMatrixLevelService riskAssessmentMatrixLevelService;

    @Resource
    private IEvaluationModelInfoService evaluationModelInfoService;

    @Resource
    private IEvaluationFactorItemsService evaluationFactorItemsService;

    @Resource
    private IRiskInherentListService riskInherentListService;

    @Resource
    private IRiskSignRecordService riskSignRecordService;

    @Resource
    private ISysUserService userService;


    /**
     * 服务器地址
     */
    @Value("${ruoyi.serverAddress}")
    private String serverAddress;

    /**
     * 获取自身部门及以下的项目
     *
     * @param ledgerProject 查询Dto
     * @real_return {@link R<List<LedgerProject>>}
     */
    @GetMapping("/project/list")
    public TableDataInfo getProjectList(LedgerProject ledgerProject) {
        //添加自身及以下部门筛选
        ledgerProject.setDeptId(getLoginUser().getDeptId());
        startPage();
        return getDataTable(ledgerProjectService.getProjectListByDeptId(ledgerProject));
    }

    /**
     * 获取项目下的楼宇
     *
     * @param projectId 项目id
     * @param type      类型 地上、地下
     * @real_return {@link R<List<LedgerBuilding>>}
     */
    @GetMapping("/building/list/{projectId}")
    public R<List<LedgerBuilding>> getBuildingListByProjectId(@PathVariable("projectId") Long projectId, String type) {
        return R.ok(ledgerBuildingService.getBuildingListByProjectId(projectId, type));
    }

    /**
     * 获取全部组织机构列表
     *
     * @real_return {@link R<List<SysDept>>}
     */
    @GetMapping("/dept/list")
    public R<List<SysDept>> getDeptList() {
        return R.ok(sysDeptService.selectDeptList(new SysDept()));
    }

    /**
     * 获取组织机构下的人员
     *
     * @param deptId 组织机构id
     * @real_return {@link R<List<SysUser>>}
     */
    @GetMapping("/user/list/{deptId}")
    public R<List<SysUser>> getUserListByProjectId(@PathVariable("deptId") Long deptId) {
        return R.ok(ledgerProjectService.getUserListByDeptId(deptId));
    }

    /**
     * 新建计划
     *
     * @param riskPlan 项目id
     * @real_return {@link R<List<LedgerProject>>}
     */
    @PostMapping()
    public R<Integer> saveRiskPlan(@RequestBody RiskPlan riskPlan) {
        riskPlan.setApproveUserId(getUserId());
        riskPlan.setCreateUserId(getUserId());
        riskPlan.setCreateDeptId(getDeptId());
        return R.ok(riskPlanService.saveRiskPlan(riskPlan));
    }

    /**
     * 编辑计划
     *
     * @param riskPlan 项目id
     * @real_return {@link R<List<LedgerProject>>}
     */
    @PutMapping()
    public R<Integer> updateRiskPlan(@RequestBody RiskPlan riskPlan) {
        riskPlan.setApproveUserId(getUserId());
        riskPlan.setCreateUserId(getUserId());
        riskPlan.setCreateDeptId(getDeptId());
        return R.ok(riskPlanService.updateRiskPlanById(riskPlan));
    }

    /**
     * 删除计划
     *
     * @param id 任务id
     * @real_return {@link R<List<LedgerProject>>}
     */
    @DeleteMapping("/{id}")
    public R<Integer> deleteById(@PathVariable Long id) {
        return R.ok(riskPlanService.revokeByPlanId(id, getUserId()));
    }

    /**
     * 任务管理列表
     *
     * @param status 状态
     * @real_return {@link R<List<RiskPlanAppListDto>>}
     */
    @GetMapping("/create/list")
    public R<List<RiskPlanAppListDto>> getRiskPlanAppListDtoListByCreateUser(String status, String name) {
        List<RiskPlanStatusType> statusList = new ArrayList<>();
        String state = "";
        if ("wait".equals(status)) {
            statusList = Arrays.asList(RiskPlanStatusType.CREATE_TASK, RiskPlanStatusType.WAIT_START);
            state = "待执行";
        } else if ("running".equals(status)) {
            statusList = Arrays.asList(RiskPlanStatusType.RUNNING, RiskPlanStatusType.WAIT_RETURN, RiskPlanStatusType.WAIT_LEADER_RETURN);
            state = "执行中";
        } else if ("finish".equals(status)) {
            statusList = Arrays.asList(RiskPlanStatusType.WAIT_LEADER_APPROVE, RiskPlanStatusType.WAIT_APPROVE
                    , RiskPlanStatusType.PROJECT_REPORT, RiskPlanStatusType.FINISH);
            state = "已执行";
        }
        List<RiskPlanAppListDto> list = riskPlanService.getRiskPlanAppListDtoList(statusList, getUserId(), null, null, name);
        String finalState = state;
        list.forEach(dto -> {
            dto.setState(finalState);
        });
        return R.ok(list);
    }

    /**
     * 任务执行情况列表
     *
     * @param status 状态 wait 待评估，running 执行中，finish 已完成
     * @real_return {@link R<List<RiskPlanAppListDto>>}
     */
    @GetMapping("/allocation/list")
    public R<List<RiskPlanAppListDto>> getAllocationRiskPlanAppListDtoListByCreateUser(String status, String name) {
        List<RiskPlanStatusType> statusList = new ArrayList<>();
        String state = "";
        if ("wait".equals(status)) {
            statusList = Arrays.asList(RiskPlanStatusType.CREATE_TASK, RiskPlanStatusType.WAIT_START);
            state = "待评估";
        } else if ("running".equals(status)) {
            statusList = Arrays.asList(RiskPlanStatusType.RUNNING, RiskPlanStatusType.WAIT_RETURN, RiskPlanStatusType.WAIT_LEADER_RETURN);
            state = "执行中";
        } else if ("finish".equals(status)) {
            statusList = Arrays.asList(RiskPlanStatusType.WAIT_LEADER_APPROVE, RiskPlanStatusType.WAIT_APPROVE
                    , RiskPlanStatusType.PROJECT_REPORT, RiskPlanStatusType.FINISH);
            state = "已完成";
        }
        List<RiskPlanAppListDto> list = riskPlanService.getRiskPlanAppListDtoList(statusList, null, getUserId(), null, name);
        String finalState = state;
        list.forEach(dto -> {
            Long finishNumber = riskPlanUserBuildingService.countStatusByPlanIdAndStatus(dto.getId(), "finish");
            Long totalNumber = riskPlanUserBuildingService.countStatusByPlanIdAndStatus(dto.getId(), null);

            dto.setState(finalState + "(" + finishNumber + "/" + totalNumber + ")");
        });
        return R.ok(list);
    }

    /**
     * 任务执行情况-楼栋详情列表
     *
     * @param planId 任务id
     * @real_return {@link R<List<RiskPlanAppListDto>>}
     */
    @GetMapping("/allocation/building/list/{planId}")
    public R<List<RiskPlanAppBuildingListDto>> getRiskPlanAppBuildingListDtoByPlanId(@PathVariable Long planId) {
        return R.ok(riskPlanUserBuildingService.getRiskPlanAppBuildingListDtoByPlanId(planId));
    }

    /**
     * 风险评估列表
     *
     * @param status 状态 running 待执行，finish 已执行，return 已退回
     * @real_return {@link R<List<RiskPlanAppListDto>>}
     */
    @GetMapping("/task/list")
    public R<List<RiskPlanAppListDto>> getRiskPlanAppListDtoListByLeaderUser(String status, String name) {
        List<RiskPlanStatusType> statusList = new ArrayList<>();
        String state = "";
        if ("running".equals(status)) {
            statusList = Arrays.asList(RiskPlanStatusType.CREATE_TASK, RiskPlanStatusType.WAIT_START, RiskPlanStatusType.RUNNING);
            state = "待执行";
        } else if ("finish".equals(status)) {
            statusList = Arrays.asList(RiskPlanStatusType.WAIT_LEADER_APPROVE, RiskPlanStatusType.WAIT_APPROVE, RiskPlanStatusType.WAIT_LEADER_RETURN
                    , RiskPlanStatusType.PROJECT_REPORT, RiskPlanStatusType.FINISH);
            state = "已执行";
        } else if ("return".equals(status)) {
            statusList = Arrays.asList(RiskPlanStatusType.WAIT_RETURN);
            state = "已退回";
        }

        List<RiskPlanAppListDto> list = riskPlanService.getRiskPlanAppListDtoList(statusList, null, null, getUserId(), name);
        String finalState = state;
        list.forEach(dto -> {
            Long finishNumber = riskPlanUserBuildingService.countStatusByPlanIdAndStatus(dto.getId(), "finish");
            Long totalNumber = riskPlanUserBuildingService.countStatusByPlanIdAndStatus(dto.getId(), null);

            dto.setState(finalState + "(" + finishNumber + "/" + totalNumber + ")");
        });
        return R.ok(list);
    }

    /**
     * 获取任务单详情
     *
     * @param planId 计划id
     * @real_return {@link R<RiskPlanAppDetailsDto>}
     */
    @GetMapping("/details/{planId}")
    public R<RiskPlanAppDetailsDto> getRiskPlanAppDetailsDtoByPlanId(@PathVariable("planId") Long planId, Long buildingId) {
        return R.ok(riskPlanService.getRiskPlanAppDetailsDtoByPlanId(planId, buildingId));
    }

    /**
     * 获取计划下的楼宇
     *
     * @param planId 项目id
     * @real_return {@link R<List<LedgerBuilding>>}
     */
    @GetMapping("/plan/building/list/{planId}")
    public R<List<LedgerBuilding>> getBuildingListByPlanId(@PathVariable("planId") Long planId) {
        return R.ok(riskPlanService.getBuildingListByPlanId(planId));
    }

    /**
     * 根据楼宇id获取楼层列表
     *
     * @param buildingId 楼宇id
     * @real_return {@link R<List<LedgerFloor>>}
     */
    @GetMapping("/floor/list/{buildingId}")
    public R<List<LedgerFloorDto>> getFloorListByBuildingId(@PathVariable("buildingId") Long buildingId) {
        return R.ok(ledgerFloorService.getFloorListByBuildingId(buildingId));
    }

    /**
     * 根据楼层id获取房间列表
     *
     * @param floorId 楼宇id
     * @real_return {@link R<List<LedgerRoom>>}
     */
    @GetMapping("/room/list/{floorId}")
    public R<List<LedgerRoom>> getRoomListByFloorId(@PathVariable("floorId") Long floorId) {
        return R.ok(ledgerRoomService.getRoomListByFloorId(floorId));
    }

    /**
     * 根据计划id、楼宇id、楼层id、房间id获取固有风险列表
     *
     * @param planId     计划id
     * @param buildingId 楼宇id
     * @param floorId    楼层id
     * @param roomId     房间id
     * @real_return {@link R<List<RiskPlanAppInherentListDto>>}
     */
    @GetMapping("/inherent/list/{planId}/{buildingId}")
    public R<List<RiskPlanAppInherentListDto>> getInherentListByPlanId(@PathVariable("planId") Long planId
            , @PathVariable("buildingId") Long buildingId, Long floorId, Long roomId) {
        return R.ok(riskPlanInherentListService.getInherentListByPlanId(null, buildingId, floorId, roomId));
    }

    /**
     * 根据计划id、楼宇id、楼层id、房间id获取固有风险列表
     *
     * @param planId 计划id
     * @real_return {@link R<List<RiskPlanAppInherentListDto>>}
     */
    @GetMapping("/inherent/list/{planId}")
    public R<List<RiskPlanAppInherentListDto>> getInherentListByPlanId(@PathVariable("planId") Long planId) {
        RiskPlan riskPlan = riskPlanService.selectRiskPlanById(planId);
        return R.ok(riskPlanInherentListService.getInherentListByPlanId(riskPlan.getProjectId(), null, null, null));
    }

    /**
     * 添加固有风险
     *
     * @param saveDto 固有风险表单
     * @real_return {@link R<Boolean>}
     */
    @PostMapping("/inherent")
    public R<Boolean> saveRiskInherentListSaveDto(RiskInherentListSaveDto saveDto
            , @RequestParam(value = "pictureFile[]", required = false) MultipartFile[] pictureFile
            , @RequestParam(value = "measuresProjectFile[]", required = false) MultipartFile[] measuresProjectFile
            , @RequestParam(value = "measuresAdministrationFile[]", required = false) MultipartFile[] measuresAdministrationFile
            , @RequestParam(value = "measuresEmergencyFile[]", required = false) MultipartFile[] measuresEmergencyFile) {

        String userId = getUserId().toString();

        //风险源照片
        if ("null".equals(saveDto.getPictureFileIds())) {
            saveDto.setPictureFileIds(null);
        }
        if (pictureFile != null) {
            String pictureFileIds = Arrays.stream(pictureFile)
                    .filter(multipartFile -> !multipartFile.isEmpty())
                    .map(multipartFile -> uploadFileService.upload(multipartFile, RuoYiConfig.getUploadPath(), userId, serverAddress).toString())
                    .collect(Collectors.joining(","));
            if (StringUtils.isNotEmpty(saveDto.getPictureFileIds())) {//编辑信息附件存在拼接
                pictureFileIds = saveDto.getPictureFileIds() + "," + pictureFileIds;
            }
            saveDto.setPictureFileIds(pictureFileIds);
        }

        //工程技术措施附件
        if ("null".equals(saveDto.getMeasuresProjectFileIds())) {
            saveDto.setMeasuresProjectFileIds(null);
        }
        if (measuresProjectFile != null) {
            String measuresProjectFileIds = Arrays.stream(measuresProjectFile)
                    .filter(multipartFile -> !multipartFile.isEmpty())
                    .map(multipartFile -> uploadFileService.upload(multipartFile, RuoYiConfig.getUploadPath(), userId, serverAddress).toString())
                    .collect(Collectors.joining(","));
            if (StringUtils.isNotEmpty(saveDto.getMeasuresProjectFileIds())) {//编辑信息附件存在拼接
                measuresProjectFileIds = saveDto.getMeasuresProjectFileIds() + "," + measuresProjectFileIds;
            }
            saveDto.setMeasuresProjectFileIds(measuresProjectFileIds);
        }

        //管理措施附件
        if ("null".equals(saveDto.getMeasuresAdministrationFileIds())) {
            saveDto.setMeasuresAdministrationFileIds(null);
        }
        if (measuresAdministrationFile != null) {
            String measuresAdministrationFileIds = Arrays.stream(measuresAdministrationFile)
                    .filter(multipartFile -> !multipartFile.isEmpty())
                    .map(multipartFile -> uploadFileService.upload(multipartFile, RuoYiConfig.getUploadPath(), userId, serverAddress).toString())
                    .collect(Collectors.joining(","));
            if (StringUtils.isNotEmpty(saveDto.getMeasuresAdministrationFileIds())) {//编辑信息附件存在拼接
                measuresAdministrationFileIds = saveDto.getMeasuresAdministrationFileIds() + "," + measuresAdministrationFileIds;
            }
            saveDto.setMeasuresAdministrationFileIds(measuresAdministrationFileIds);
        }

        //应急处置措施附件
        if ("null".equals(saveDto.getMeasuresEmergencyFileIds())) {
            saveDto.setMeasuresEmergencyFileIds(null);
        }
        if (measuresEmergencyFile != null) {
            String measuresEmergencyFileIds = Arrays.stream(measuresEmergencyFile)
                    .filter(multipartFile -> !multipartFile.isEmpty())
                    .map(multipartFile -> uploadFileService.upload(multipartFile, RuoYiConfig.getUploadPath(), userId, serverAddress).toString())
                    .collect(Collectors.joining(","));
            if (StringUtils.isNotEmpty(saveDto.getMeasuresEmergencyFileIds())) {//编辑信息附件存在拼接
                measuresEmergencyFileIds = saveDto.getMeasuresEmergencyFileIds() + "," + measuresEmergencyFileIds;
            }
            saveDto.setMeasuresEmergencyFileIds(measuresEmergencyFileIds);
        }

        return R.ok(riskPlanInherentListService.saveRiskInherentListSaveDto(saveDto));
    }

    /**
     * 获取固有风险详情
     *
     * @param inherentId 固有风险id
     * @real_return {@link R<RiskPlanAppInherentListDetailsDto>}
     */
    @GetMapping("/inherent/details/{inherentId}")
    public R<RiskPlanAppInherentListDetailsDto> getRiskPlanAppInherentListDetailsDtoByInherentId(@PathVariable("inherentId") Long inherentId) {
        return R.ok(riskPlanInherentListService.getRiskPlanAppInherentListDetailsDtoByInherentId(inherentId));
    }

    /**
     * 获取固有风险删除
     *
     * @param inherentId 固有风险id
     * @real_return {@link R<Boolean>}
     */
    @GetMapping("/inherent/delete/{inherentId}")
    public R<Boolean> deleteRiskPlanAppInherentByInherentId(@PathVariable("inherentId") Long inherentId) {
        RiskPlanInherentList riskPlanInherentList = new RiskPlanInherentList();
        riskPlanInherentList.setId(inherentId);
        riskPlanInherentList.setDelFlag("1");
        return R.ok(riskPlanInherentListService.updateRiskPlanInherentList(riskPlanInherentList) > 0);
    }

    /**
     * 根据固有风险id获取现有风险列表
     *
     * @param inherentId 固有风险id
     * @real_return {@link R<List<RiskPlanAppExistingListDto>>}
     */
    @GetMapping("/existing/list/{inherentId}")
    public R<List<RiskPlanAppExistingListDto>> getExistingListByInherentId(@PathVariable("inherentId") Long inherentId) {
        return R.ok(riskPlanInherentListService.getExistingListByInherentId(inherentId));
    }


    /**
     * 添加现有风险
     *
     * @param saveDto 现有风险表单
     * @real_return {@link R<Boolean>}
     */
    @PostMapping("/existing")
    public R<Boolean> saveRiskExistingListSaveDto(RiskExistingListSaveDto saveDto
            , @RequestParam(value = "pictureFile[]", required = false) MultipartFile[] pictureFile
            , @RequestParam(value = "measuresProjectFile[]", required = false) MultipartFile[] measuresProjectFile
            , @RequestParam(value = "measuresAdministrationFile[]", required = false) MultipartFile[] measuresAdministrationFile
            , @RequestParam(value = "measuresEmergencyFile[]", required = false) MultipartFile[] measuresEmergencyFile) {

        String userId = getUserId().toString();

        //风险源照片
        if ("null".equals(saveDto.getPictureFileIds())) {
            saveDto.setPictureFileIds(null);
        }
        if (pictureFile != null) {
            String pictureFileIds = Arrays.stream(pictureFile)
                    .filter(multipartFile -> !multipartFile.isEmpty())
                    .map(multipartFile -> uploadFileService.upload(multipartFile, RuoYiConfig.getUploadPath(), userId, serverAddress).toString())
                    .collect(Collectors.joining(","));
            if (StringUtils.isNotEmpty(saveDto.getPictureFileIds())) {//编辑信息附件存在拼接
                pictureFileIds = saveDto.getPictureFileIds() + "," + pictureFileIds;
            }
            saveDto.setPictureFileIds(pictureFileIds);
        }

        //工程技术措施附件
        if ("null".equals(saveDto.getMeasuresProjectFileIds())) {
            saveDto.setMeasuresProjectFileIds(null);
        }
        if (measuresProjectFile != null) {
            String measuresProjectFileIds = Arrays.stream(measuresProjectFile)
                    .filter(multipartFile -> !multipartFile.isEmpty())
                    .map(multipartFile -> uploadFileService.upload(multipartFile, RuoYiConfig.getUploadPath(), userId, serverAddress).toString())
                    .collect(Collectors.joining(","));
            if (StringUtils.isNotEmpty(saveDto.getMeasuresProjectFileIds())) {//编辑信息附件存在拼接
                measuresProjectFileIds = saveDto.getMeasuresProjectFileIds() + "," + measuresProjectFileIds;
            }
            saveDto.setMeasuresProjectFileIds(measuresProjectFileIds);
        }

        //管理措施附件
        if ("null".equals(saveDto.getMeasuresAdministrationFileIds())) {
            saveDto.setMeasuresAdministrationFileIds(null);
        }
        if (measuresAdministrationFile != null) {
            String measuresAdministrationFileIds = Arrays.stream(measuresAdministrationFile)
                    .filter(multipartFile -> !multipartFile.isEmpty())
                    .map(multipartFile -> uploadFileService.upload(multipartFile, RuoYiConfig.getUploadPath(), userId, serverAddress).toString())
                    .collect(Collectors.joining(","));
            if (StringUtils.isNotEmpty(saveDto.getMeasuresAdministrationFileIds())) {//编辑信息附件存在拼接
                measuresAdministrationFileIds = saveDto.getMeasuresAdministrationFileIds() + "," + measuresAdministrationFileIds;
            }
            saveDto.setMeasuresAdministrationFileIds(measuresAdministrationFileIds);
        }

        //应急处置措施附件
        if ("null".equals(saveDto.getMeasuresEmergencyFileIds())) {
            saveDto.setMeasuresEmergencyFileIds(null);
        }
        if (measuresEmergencyFile != null) {
            String measuresEmergencyFileIds = Arrays.stream(measuresEmergencyFile)
                    .filter(multipartFile -> !multipartFile.isEmpty())
                    .map(multipartFile -> uploadFileService.upload(multipartFile, RuoYiConfig.getUploadPath(), userId, serverAddress).toString())
                    .collect(Collectors.joining(","));
            if (StringUtils.isNotEmpty(saveDto.getMeasuresEmergencyFileIds())) {//编辑信息附件存在拼接
                measuresEmergencyFileIds = saveDto.getMeasuresEmergencyFileIds() + "," + measuresEmergencyFileIds;
            }
            saveDto.setMeasuresEmergencyFileIds(measuresEmergencyFileIds);
        }

        Map<String, String> mapResult = riskPlanExistingListService.getRiskLevelByScoreAndUpd(saveDto.getBuildingId(), saveDto.getLevelScore());
        saveDto.setLevel(mapResult.get("level"));
        saveDto.setRiskColor(mapResult.get("riskColor"));
        return R.ok(riskPlanExistingListService.saveRiskExistingListSaveDto(saveDto));
    }


    /**
     * 获取现有风险详情
     *
     * @param existingId 现有风险风id
     * @real_return {@link R<RiskPlanAppExistingListDetailsDto>}
     */
    @GetMapping("/existing/details/{existingId}")
    public R<RiskPlanAppExistingListDetailsDto> getRiskPlanAppExistingListDetailsDtoByExistingId(@PathVariable("existingId") Long existingId) {
        return R.ok(riskPlanExistingListService.getRiskPlanAppExistingListDetailsDtoByExistingId(existingId));
    }

    /**
     * 获取现有风险删除
     *
     * @param existingId 现有风险id
     * @real_return {@link R<Boolean>}
     */
    @GetMapping("/existing/delete/{existingId}")
    public R<Boolean> deleteRiskPlanAppExistingByExistingId(@PathVariable("existingId") Long existingId) {
        RiskPlanExistingList existingList = new RiskPlanExistingList();
        existingList.setId(existingId);
        existingList.setDelFlag("1");
        return R.ok(riskPlanExistingListService.updateById(existingList));
    }

    /**
     * 风险评估-任务完成
     *
     * @param planId     任务id
     * @param buildingId 楼宇id
     * @real_return {@link R<Integer>}
     */
    @PostMapping("/submit/{planId}/{buildingId}")
    public R<Integer> submitByPlanIdAndBuildingId(@PathVariable Long planId, @PathVariable Long buildingId) {
        return R.ok(riskPlanUserBuildingService.submitByPlanIdAndBuildingId(planId, buildingId, getUserId()));
    }


    /**
     * 根据风险任务审批列表
     *
     * @param status 状态 running 待审批 completed 已审批
     * @real_return {@link R<List<RiskPlanAppExistingListDto>>}
     */
    @GetMapping("/approve/list")
    public R<List<RiskPlanAppListDto>> getRiskPlanAppListDtoListByApproveUser(String status) {
        String result = null;
        if ("completed".equals(status)) {
            status = "completed";
            result = "1";
        } else if ("return".equals(status)) {
            status = "completed";
            result = "0";
        }
        return R.ok(riskPlanService.getRiskPlanAppListDtoListByApproveUser(getUserId(), status, result));
    }


    /**
     * 根据风险任务审批
     *
     * @real_return {@link R<Integer>}
     */
    @PostMapping("/approve")
    public R<Integer> approveAppApproveDto(@RequestBody RiskPlanAppApproveDto appApproveDto) {
        appApproveDto.setUserId(getUserId());
        return R.ok(riskPlanService.approveAppApproveDto(appApproveDto));
    }


    /**
     * 退回删除接口
     *
     * @real_return {@link R<Integer>}
     */
    @PostMapping("/revoke/{planId}")
    public R<Integer> revokeByPlanId(@PathVariable("planId") Long planId) {
        return R.ok(riskPlanService.revokeByPlanId(planId, getUserId()));
    }


    /**
     * 获取法律依据
     *
     * @real_return {@link R<List<LawList>>}
     */
    @GetMapping("/law/list")
    public R<List<LawList>> getLawList(LawList lawList) {
        return R.ok(lawListService.selectLawListList(lawList));
    }


    /**
     * 根据字典key获取字典列表
     *
     * @real_return {@link R<List<SysDictData>>}
     */
    @GetMapping("/dict/data/list")
    public R<List<SysDictData>> getSysDictDataList(SysDictData dictData) {
        return R.ok(dictDataService.selectDictDataList(dictData));
    }

    /**
     * 根据横纵坐标获取对应可能性或严重性列表
     *
     * @param type 类型 possibility 可能性 severity 严重性
     * @real_return {@link R<List<RiskAssessmentMatrixScore>>}
     */
    @GetMapping("/matrix/{type}/list")
    public R<List<RiskAssessmentMatrixScore>> getRiskAssessmentMatrixScoreListByType(@PathVariable String type) {
        return R.ok(riskAssessmentMatrixScoreService.getRiskAssessmentMatrixScoreListByType(type));
    }

    /**
     * 根据得分返回对应的风险等级
     *
     * @param score 得分
     * @real_return {@link R<RiskAssessmentMatrixLevel>}
     */
    @GetMapping("/matrix/score/{score}")
    public R<RiskAssessmentMatrixLevel> getRiskAssessmentMatrixLevelByScore(@PathVariable BigDecimal score) {
        return R.ok(riskAssessmentMatrixLevelService.getRiskAssessmentMatrixLevelByScore(score));
    }

    /**
     * 任务台账列表
     *
     * @real_return {@link R<List<RiskPlanAppListDto>>}
     */
    @GetMapping("/details/list")
    public R<List<RiskPlanAppListDto>> getRiskPlanAppListDtoListByCreateUserDeptId(String name) {
        Long deptId = getLoginUser().getDeptId();
        List<RiskPlanAppListDto> list = riskPlanService.getRiskPlanAppListDtoListByCreateUserDeptId(deptId, name);
        list.forEach(dto -> {
            dto.setState(RiskPlanStatusType.getTitleByStatus(dto.getStatus()).getTitle());
        });
        return R.ok(list);
    }

    /**
     * 返回责单位列表
     */
    @GetMapping("/inherent/getResponsibilityUnits")
    public R<List<RiskPlanInherentList>> getResponsibilityUnits(String unitName, String projectId) {

        List<RiskPlanInherentList> list = new ArrayList<>();
        QueryWrapper<RiskPlan> query = new QueryWrapper<>();
        query.eq("project_id", projectId);
        List<RiskPlan> listPlan = riskPlanService.list(query);
        if (listPlan != null && listPlan.size() > 0) {
            List<Long> listPlanIds = new ArrayList<>();
            for (RiskPlan riskPlan : listPlan) {
                listPlanIds.add(riskPlan.getId());
            }
            QueryWrapper<RiskPlanInherentList> queryWrapper = new QueryWrapper<>();
            queryWrapper.select("measures_dept_name").in("plan_id", listPlanIds).like("measures_dept_name", unitName).isNotNull("measures_dept_name").groupBy("measures_dept_name");
            list = riskPlanInherentListService.list(queryWrapper);
        }

        return R.ok(list);
    }

    /**
     * 返回责单位列表
     */
    @GetMapping("/inherent/getResponsibilityPersons")
    public R<List<RiskPlanInherentList>> getResponsibilityPersons(String personName, String projectId) {

        List<RiskPlanInherentList> list = new ArrayList<>();
        QueryWrapper<RiskPlan> query = new QueryWrapper<>();
        query.eq("project_id", projectId);
        List<RiskPlan> listPlan = riskPlanService.list(query);
        if (listPlan != null && listPlan.size() > 0) {
            List<Long> listPlanIds = new ArrayList<>();
            for (RiskPlan riskPlan : listPlan) {
                listPlanIds.add(riskPlan.getId());
            }
            QueryWrapper<RiskPlanInherentList> queryWrapper = new QueryWrapper<>();
            queryWrapper.select("measures_user_name", "measures_user_phone").in("plan_id", listPlanIds).like("measures_user_name", personName).isNotNull("measures_user_name").groupBy("measures_user_name");
            list = riskPlanInherentListService.list(queryWrapper);
        }
        return R.ok(list);
    }

    /**
     * app端新建项目数据
     */
    @Log(title = "app端新建项目数据", businessType = BusinessType.INSERT)
    @PostMapping("/project")
    public AjaxResult addProject(@RequestBody LedgerProject ledgerProject) {
        ledgerProject.setCreateBy(getUsername());
        ledgerProjectService.insertLedgerProject(ledgerProject);
        return AjaxResult.success(ledgerProject.getId());
    }

    /**
     * app端新建楼宇数据
     */
    @Log(title = "app端新建楼宇数据", businessType = BusinessType.INSERT)
    @PostMapping("/building")
    public AjaxResult addBuilding(@RequestBody LedgerBuilding ledgerBuilding) {
        ledgerBuilding.setCreateBy(getUsername());
        ledgerBuilding.setFloor(ledgerBuilding.getGroundFloor() + ledgerBuilding.getUndergroundFloor() + "");
        int result = ledgerBuildingService.insertLedgerBuilding(ledgerBuilding);
        //导入楼层信息
        if (ledgerBuilding.getGroundFloor() != 0) {
            for (int i = 1; i <= ledgerBuilding.getGroundFloor(); i++) {
                LedgerFloor ledgerFloor = new LedgerFloor();
                ledgerFloor.setBuildingId(ledgerBuilding.getId());
                ledgerFloor.setName(i + "层");
                ledgerFloor.setType("地上");
                ledgerFloorService.insertLedgerFloor(ledgerFloor);
            }
        }
        if (ledgerBuilding.getUndergroundFloor() != 0) {
            for (int i = 1; i <= ledgerBuilding.getUndergroundFloor(); i++) {
                LedgerFloor ledgerFloor = new LedgerFloor();
                ledgerFloor.setBuildingId(ledgerBuilding.getId());
                ledgerFloor.setName(-i + "层");
                ledgerFloor.setType("地下");
                ledgerFloorService.insertLedgerFloor(ledgerFloor);
            }
        }
        return toAjax(result);
    }

    /**
     * 根据人机料法环获取
     *
     * @param factorNames 影响因素
     * @return
     */
    @GetMapping("/evaluation/getFactorListByType")
    public R<List<FactorListDto>> getFactorListByType(String factorNames) {
        /**
         * 解决思路
         * 1.获取影响因素
         * 2.根据影响因素获取印象因子
         * 3.获取影响因子选项
         */

        List<FactorListDto> list_rs = new ArrayList<>();
        List<FactorItemListDto> list_items = new ArrayList<>();
        FactorListDto factorListDto = new FactorListDto();
        FactorItemListDto factorItemListDto = new FactorItemListDto();
        if (factorNames != null && !factorNames.trim().equals("")) {

            List<String> list_factors = Arrays.asList(factorNames.split(","));
            for (int i = 0; i < list_factors.size(); i++) {
                factorListDto = new FactorListDto();
                list_items = new ArrayList<>();
                String factorName = list_factors.get(i);
                QueryWrapper<EvaluationModelInfo> queryModel = new QueryWrapper<>();
                queryModel.lambda().eq(EvaluationModelInfo::getInfluenceFactor, factorName);
                List<EvaluationModelInfo> list_models = evaluationModelInfoService.list(queryModel);
                if (list_models.size() > 0) {

                    for (EvaluationModelInfo modelInfo : list_models) {
                        factorItemListDto = new FactorItemListDto();
                        QueryWrapper<EvaluationFactorItems> queryItem = new QueryWrapper<>();
                        queryItem.lambda().eq(EvaluationFactorItems::getFactorId, modelInfo.getId());
                        List<EvaluationFactorItems> itemsList = evaluationFactorItemsService.list(queryItem);
                        factorItemListDto.setFactorSubName(modelInfo.getInfluenceSubFactor());
                        factorItemListDto.setRemark(modelInfo.getRemark());
                        factorItemListDto.setFactorItemsList(itemsList);
                        list_items.add(factorItemListDto);

                    }
                }
                factorListDto.setFactorName(factorName);
                factorListDto.setFactorList(list_items);
                factorListDto.setFactorAmount(list_items.size());
                list_rs.add(factorListDto);


            }


        }

        return R.ok(list_rs);
    }

    /**
     * app端新建楼层数据
     */
    @Log(title = "app端新建楼层数据", businessType = BusinessType.INSERT)
    @PostMapping("/floor")
    public AjaxResult addFloor(@RequestBody LedgerFloor ledgerFloor) {
        ledgerFloor.setCreateBy(getUsername());
        ledgerFloorService.insertLedgerFloor(ledgerFloor);
        return AjaxResult.success(ledgerFloor.getId());
    }

    /**
     * app端复制楼层下全部数据
     */
    @Log(title = "app端复制楼层下全部数据", businessType = BusinessType.INSERT)
    @PostMapping("/copy/floor/{planId}/{sourceFloorId}/{targetFloorId}")
    public AjaxResult copyFloor(@PathVariable Long planId
            , @PathVariable Long sourceFloorId, @PathVariable Long targetFloorId) {

        return AjaxResult.success(ledgerFloorService.copyFloor(sourceFloorId, targetFloorId, planId, getLoginUser()));
    }


    /**
     * 根据分数获取风险等级，并计算楼栋风险值
     *
     * @param buildingId
     * @param score
     * @return
     */
    @GetMapping("/existing/matrix/score")
    public R<Map<String, String>> getRiskLevelByScore(Long buildingId, BigDecimal score) {
//        List<LedgerBuilding> list = ledgerBuildingService.list();
//        for (LedgerBuilding item : list) {
//            riskPlanExistingListService.getRiskLevelByScore(item.getId(), new BigDecimal("4.5"));
//
//        }
        return R.ok(riskPlanExistingListService.getRiskLevelByScore(buildingId, score));
    }

    /**
     * 获取固有风险模版列表
     */
    @GetMapping("/inherent/template/list")
    public TableDataInfo list(RiskInherentListVO riskInherentList) {
        startPage();
        return getDataTable(riskInherentListService.selectRiskInherentListListVO(riskInherentList));
    }


    /**
     * app端复制房间
     */
    @Log(title = "app端复制房间", businessType = BusinessType.INSERT)
    @PostMapping("/copy/room/{planId}")
    public AjaxResult copyRoom(@RequestBody LedgerRoom ledgerRoom, @PathVariable Long planId) {
        return AjaxResult.success(ledgerRoomService.copyRoom(ledgerRoom, getLoginUser(), planId));
    }


    /**
     * 提交签到签退记录
     */
    @Log(title = "提交签到签退记录", businessType = BusinessType.INSERT)
    @PostMapping("/sign")
    public AjaxResult saveSign(RiskSignRecord riskSignRecord
            , @RequestParam(value = "file[]", required = false) MultipartFile[] file) {
        //判断当前人、当前项目今天是否加班
        if (!riskSignRecordService.checkByUserIdAndProjectIdDate(riskSignRecord.getUserId(), riskSignRecord.getProjectId()
                , riskSignRecord.getType(), new Date())) {
            return AjaxResult.error("当前项目今天您已提交" + (riskSignRecord.getType() ? "签到" : "签退") + "记录！");
        }
        riskSignRecord.setCreateBy(getLoginUser().getUsername());
        riskSignRecord.setCreateTime(new Date());
        if (file != null) {
            String userId = getUserId() + "";
            String pictureFileIds = Arrays.stream(file)
                    .filter(multipartFile -> !multipartFile.isEmpty())
                    .map(multipartFile -> uploadFileService.upload(multipartFile, RuoYiConfig.getUploadPath(), userId, serverAddress).toString())
                    .collect(Collectors.joining(","));
            riskSignRecord.setFileIds(pictureFileIds);
        }
        return AjaxResult.success(riskSignRecordService.save(riskSignRecord));
    }


    /**
     * 获取固有风险模版列表
     */
    @GetMapping("/sign/list")
    public TableDataInfo getSignList(RiskSignRecord riskSignRecord) {
        startPage();
        riskSignRecord.setUserId(getUserId());
        List<RiskSignRecord> signList = riskSignRecordService.getSignList(riskSignRecord);
        TableDataInfo dataTable = getDataTable(signList);
        List<RiskSignRecordAppShowDto> showDtos = signList.stream().map(dto -> {
            RiskSignRecordAppShowDto showDto = new RiskSignRecordAppShowDto();
            //风险源照片
            BeanUtils.copyProperties(dto, showDto);
            if (!StrUtil.hasEmpty(dto.getFileIds())) {
                List<Long> fileIds = Arrays.stream(dto.getFileIds().split(","))
                        .map(Long::parseLong)
                        .collect(Collectors.toList());
                showDto.setFiles(uploadFileService.selectSysFileByFileIds(fileIds));
            }
            //人员
            if (dto.getUserId() != null) {
                SysUser sysUser = userService.selectUserById(dto.getUserId());
                showDto.setUserName(sysUser != null ? sysUser.getNickName() : null);
            }
            //项目
            if (dto.getProjectId() != null) {
                LedgerProject ledgerProject = ledgerProjectService.selectLedgerProjectById(dto.getProjectId());
                showDto.setProjectName(ledgerProject != null ? ledgerProject.getName() : null);
            }
            return showDto;
        }).collect(Collectors.toList());
        dataTable.setRows(showDtos);
        return dataTable;
    }

}
